use std::thread;
use std::time::Duration;

use evdev_rs::util::event_code_to_int;

use crossbeam::crossbeam_channel::{unbounded, Select, Receiver, Sender};

use super::event::ThreadInputEvent;
use super::hw_device;

pub fn create_evdev_thread(device_path: &str) -> (String, Receiver<ThreadInputEvent>, Sender<bool>) {
    // send out input events, receive stop command
    let (send_event_channel, recv_event_channel) = unbounded();
    let (send_stop_channel, recv_stop_channel) = unbounded();

    // thread safety and stuff
    let device_path = String::from(device_path);
    let return_device_path = device_path.clone();

    thread::spawn(move || {
        // open device
        let mut dev = hw_device::HwDevice::new(&device_path);
        // wait for device init
        thread::sleep(Duration::from_secs(1));
        // don't deliver events elsewhere
        dev.device.grab(evdev_rs::GrabMode::Grab).unwrap();
        // sleep for testing purposes
        thread::sleep(Duration::from_secs(1));

        loop {
            // check for stop requests
            let mut sel = Select::new();
            // create stop channel receive operation
            let oper1 = sel.recv(&recv_stop_channel);
            // try to select from stop channel
            let oper = sel.try_select();
            match oper {
                // no stop requests
                Err(_) => (),
                // stop request received, break out of the loop
                Ok(oper) => match oper.index() {
                    i if i == oper1 => {
                        assert_eq!(oper.recv(&recv_stop_channel), Ok(true));
                        break;
                    },
                    // shouldn't happen
                    _ => (),
                },
            }

            // read next input event
            let event = dev.device.next_event(evdev_rs::NORMAL | evdev_rs::BLOCKING);
            match event {
                // send input event to event channel
                Ok(e) => send_event_channel.send(
                    ThreadInputEvent {
                        typ: e.1.event_type as u32,
                        code: event_code_to_int(&e.1.event_code).1,
                        value: e.1.value,
                    }
                ).unwrap(),
                Err(_) => (),
            }
        }
    });

    (return_device_path, recv_event_channel, send_stop_channel)
}
